// Additional function libraries; see text
// These libs must be added to your Arduino Sketchbook 'libraries' folder!
#include <FatReader.h>
#include <SdReader.h>
#include "WaveUtil.h"
#include "WaveHC.h"
#include <MsTimer2.h>

// Define objects
SdReader card;
FatVolume vol;
FatReader root;
FatReader f;
WaveHC wave;

// Define constants for use elsewhere in sketch
const int EYES_LED       =   6;      // Eyes LEDs connected to pin D6
const int MIN_BRIGHT     =   10;     // PWM minimum value
const int MAX_BRIGHT     =   255;    // PWM maximum value (255=full)
const int FLICK_INTERVAL =   30;     // 30 milliseconds between "flickers"
const int PIEZO          =   14;     // Piezo disc to analog A0
const int PBTTN1         =   15;     // Pushbutton switch to analog A1
const int TRIGGER        =   550;    // Trigger value for piezo disc

// Program initialization
void setup() {
 
  // Set up a random number using "noise" on unconnected analog A5  
  randomSeed(analogRead(A5));
  
  // Set up serial port for debugging
  Serial.begin(9600);
  Serial.println("Halloween Wave Shield");
  
  // Set output pins for DAC control (hardwared on the Wave Shield)
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  
  // Set pin 13 (Arduino LED) to OUTPUT
  pinMode(13, OUTPUT);
  
  // Set eyes LED pin as an output
  pinMode(EYES_LED, OUTPUT);

  // Set A4 as output (0V); used as ground for buttons and piezo
  pinMode(A4, OUTPUT);
  digitalWrite(A4, LOW);  
 
  // Enable pull-up resistors for switches on A1-A3
  digitalWrite(A1, HIGH);    // PBTTN1
  digitalWrite(A2, HIGH);    // Pin not used in this example
  digitalWrite(A3, HIGH);    // Not not used in this example

  // Initialize SD card; if card fails init halt program
  if (!card.init()) {
    Serial.println("Card initialization failed");
    sdErrorCheck();
    while(1);
  }
  
  // Enable optimize read
  card.partialBlockRead(true);
 
// Look for FAT partition!
  uint8_t part;
  for (part = 0; part < 5; part++) {     // Look in up to 5 slots
    if (vol.init(card, part)) 
      break;                             // Slot found
  }
  // Error out if no slot found
  if (part == 5) {
    Serial.println("No a valid FAT partition");
    sdErrorCheck();
    while(1);                // Halt program
  }
  
  // Display card partition and FAT info
  Serial.print("Using partition ");
  Serial.print(part, DEC);
  Serial.print(", type is FAT");
  Serial.println(vol.fatType(),DEC);
  
  // Try to open the root directory; if error halt program
  if (!root.openRoot(vol)) {
    Serial.println("Cannot open root directory");
    while(1);
  }
  
  // Set background hardware timer to call 'flicker' function
  //  once each flicker interval
  MsTimer2::set(FLICK_INTERVAL, flicker); 
  
  Serial.println("Ready");
}

// Main program loop
void loop() {
  if(analogRead(PIEZO) > TRIGGER)
    playcomplete("1.WAV");        // Plays when piezo is trigger
    
  if(digitalRead(PBTTN1) == LOW)
    playcomplete("2.WAV");        // Plays when pushbutton is pressed
  
  delay(50);
}

// Play a file
void playcomplete(char *name) {
  // Start hardware timer for eyes flickering
  MsTimer2::start();        // Start hardware timer for eye flicker
  playfile(name);
  while (wave.isplaying) {
    // Cycle in loop while file is playing
  }
  // Done playing
  MsTimer2::stop();        // Stop hardware timer for eye flicker
  digitalWrite(EYES_LED, LOW);  // Turn off eye LEDs
}

// Finds file to play from SD card
void playfile(char *name) {
  if (wave.isplaying) {// Stop any in-progress playback
    wave.stop();
  }
  // Open file from root directory
  if (!f.open(root, name)) {
    Serial.println("Cannot not open file "); 
    Serial.print(name);
    return;
  }
  // Read file and turn it into a wave object
  if (!wave.create(f)) {
    Serial.println("Not a valid WAV"); 
    return;
  }
  
  // Begin playback
  wave.play();
}

// Validate SD card I/O
void sdErrorCheck(void) {
  if (!card.errorCode()) return;
  putstring("\n\rSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  putstring(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

// MSTimer2 hardware interrupt service routine
//   called at every timer interval, as set previously
// Random function alters brightness each time the
//   function is called
void flicker(void) {
  analogWrite(EYES_LED, random(MIN_BRIGHT, MAX_BRIGHT));
}

